/* PubScreenNames.e
 * Affiche le nom et quelques atres attributs d'crans public ouverts
 * Teste aussi la prsence d'cran particulier.
 *
 * Domaine Public par Diego Caravana.
 * ( Source traduit par Olivier ANH (BUGSS) )
 *
 * Merci  Wouter van Oortmerssen pour son TRES BON travail, en esprant
 * qu'il continuera a programmer en E !
 *
 * H, apprenez le E !!!
 *
 */

/* TAB=4 */

OPT OSVERSION=37

/* Ici les OBJECTs qu'on utilise pour copier les donnes inhrentes aux crans
 * public. Copier les donnes est ncessaire car le vrroulliage (lock) de la
 * liste (PubScreen List) doit tre le plus court possible (voir les autodocs
 * de Commodore). C'est due au fait que la liste ne peut tre modifie (i.e.
 * aucune fntre ne peut s'ouvrir ou se fermer) tant qu'elle est verrouille
 * par un tache.
 */

OBJECT mypubscreen
        name, flags:INT, visitorcnt:INT, screen, task, next
ENDOBJECT

MODULE 'intuition/screens', 'exec/lists', 'exec/nodes'

CONST ARGS_NUM=2
ENUM  ARG_FULL, ARG_EXISTS

ENUM ERR_NONE=0, ERR_PUBLOCK, ERR_MEM, ERR_WRONGARGS, OK_FOUND, OK_NOTFOUND

DEF     localpslist:mypubscreen, strflags[25]:STRING, args[ARGS_NUM]:ARRAY OF LONG

RAISE ERR_MEM           IF String()=NIL,
          ERR_PUBLOCK   IF LockPubScreenList()=NIL,
          ERR_WRONGARGS IF ReadArgs()=NIL,
          ERR_MEM               IF New()=NIL

PROC main() HANDLE
DEF pslist:PTR TO lh, psnode:PTR TO ln, i=0, localpsnode:PTR TO mypubscreen,
        pubscreen:PTR TO pubscreennode, publock=NIL, rdargs=NIL

        VOID '$VER: PubScreenNames 1.1 (13.02.93) par Diego Caravana'

        rdargs := ReadArgs('FULL/S,EXISTS/K', args, 0)

        /* Prend la liste des cran public ;
         * Elle ne sera pas modifie pendant la lecture
         */
        publock := LockPubScreenList()

        /* premire boucle : copie toutes les donnes ncessaires le plus vite possible
         */
        pslist := publock
        psnode := pslist.head
        localpsnode := localpslist

        WHILE psnode.succ <> NIL

                /* teste si un cran EXISTS avec un nom spcifi
                 */
                IF StrCmp(psnode.name, args[ARG_EXISTS], ALL) THEN Raise(OK_FOUND)

                /* alloue une chaine dynamique pour reduire la mmoire utilise
                 * car les STRINGs ne sont pas permises (bientt, j'espre :)
                 * dans les OBJECTs
                 */
                localpsnode.name:=String(StrLen(psnode.name)+2)
                StrCopy(localpsnode.name, psnode.name, ALL)
                IF args[ARG_FULL]=-1    /* copy only if needed */
                        pubscreen:=psnode
                        localpsnode.flags:=pubscreen.flags
                        localpsnode.visitorcnt:=pubscreen.visitorcount
                        localpsnode.task:=pubscreen.sigtask
                        localpsnode.screen:=pubscreen.screen
                ENDIF

                /* alloue un OBJECT pour contenir les informations du
                 * prochain cran
                 */
                localpsnode.next:=New(SIZEOF mypubscreen)

                /* change les pointeurs pour xaminer la chaine
                 */
                localpsnode:=localpsnode.next
                psnode:=psnode.succ
        ENDWHILE

        UnlockPubScreenList()
        publock:=NIL    /* pour mmoire que l'on a dverrouill (release the lock)*/

        /*  ce point, si un nom d'cran est spcifi avec EXIST, il n'a pas
         * t trouv comme Ecran Public (Public Screen) dans la boucle
         * prcdente, on peut alors sortir avec une code de retour null
         */
        IF StrLen(args[ARG_EXISTS]) <> 0 THEN Raise(OK_NOTFOUND)

        /* imprime l'entte avec la description des champs
     */
        IF args[ARG_FULL]=-1    /* imprime le bon ! */
                WriteF('\n N. Nom               Visiteurs Ecran      Tache      Flags\n')
                WriteF(  ' ---------------------------------------------------------------------------\n')
        ELSE
                WriteF('\n N. Nom\n')
                WriteF(  ' ----------------------\n')
        ENDIF

        /* Seconde boucle : imprime toutes les donnes
         */
        localpsnode:=localpslist
        WHILE localpsnode.next <> NIL
                i++

                IF args[ARG_FULL]=-1    /* choisir les infos  imprimer */

                        /* not-so-simple code: the two flags are independent by one another
                         * and also we want a "|" (OR in C) put between them; then, there
                         * is a default string which is used when no flag is set
                         */
                        StrCopy(strflags,'<No Flags Set>',ALL)
                        IF localpsnode.flags AND SHANGHAI
                                StrCopy(strflags,'SHANGHAI',ALL)
                                IF localpsnode.flags AND POPPUBSCREEN
                                        StrAdd(strflags,'|POPPUBSCREEN',ALL)
                                ENDIF
                        ELSE
                                IF localpsnode.flags AND POPPUBSCREEN
                                        StrCopy(strflags,'POPPUBSCREEN',ALL)
                                ENDIF
                        ENDIF

                        WriteF(' \l\d[2] \l\s[18]    \l\d[3]   $\z\h[8]  $\z\h[8]  \l\s\n',
                                i, localpsnode.name, localpsnode.visitorcnt,
                                localpsnode.screen, localpsnode.task, strflags )
                ELSE
                        WriteF(' \l\d[2] \l\s[18]\n', i, localpsnode.name)
                ENDIF

                localpsnode:=localpsnode.next
        ENDWHILE
        WriteF('\n Trouv \d Public Screen(s)\n\n', i)

        Raise(ERR_NONE)

EXCEPT
        IF publock THEN UnlockPubScreenList()
    IF rdargs THEN FreeArgs(rdargs)

        SELECT exception
                CASE ERR_NONE;
                CASE OK_FOUND;          CleanUp(5)              /* Si WARN, ce sera vrai (true) */
                CASE OK_NOTFOUND;       CleanUp(0)
                CASE ERR_PUBLOCK;       WriteF('*** Ne peut obtenir les infos sur PubScreen !\n')
                CASE ERR_MEM;           WriteF('*** Pas de  mmoire !\n')
                DEFAULT;                        PrintFault(IoErr(), '*** Erreur')
        ENDSELECT

        IF exception THEN CleanUp(10)
        CleanUp(0)

ENDPROC
